@@ -80,20 +80,6 @@ def skipped(iterable, n):
         else:
             yield f
             
-def chunked(iterable, n):
-    if isinstance(iterable, (list, tuple)):
-        for i in range(0, len(iterable), n):
-            yield iterable[i:i+n]
-    else:
-        chunk = []
-        for item in iterable:
-            chunk.append(item)
-            if len(lst) >= n:
-                yield chunk
-                chunk = []
-        if chunk:
-            yield chunk
-
 class MetaValidationError(Exception):
     
     def __init__(self, message, errors):
